package com.javacorporate.forum.dbobj;

/*
 * ForumMessage.java
 *
 * Copyright 1999 Javacorporate Ltd.
 */

import com.javacorporate.dbobj.*;
import com.javacorporate.db.*;
import com.javacorporate.common.dbobj.NextNumber;
import com.javacorporate.common.util.DateTime;
import java.util.Calendar;
import java.util.Hashtable;
import java.util.Enumeration;
import com.javacorporate.common.util.EventHandler;
import com.javacorporate.common.util.LogHandler;
import com.javacorporate.common.util.LogException;

/**
 * ForumMessage
 *
 * @version		$Revision: 1.6 $  $Date: 2000/06/13 12:11:45 $
 * @author		Michael Nash
 */
public class ForumMessage extends SecuredDBObject {
	private Hashtable values = null;

	/**
	 * @see com.javacorporate.dbobj.SecuredDBObject
	 */
	public ForumMessage() throws DBException {
		super();
	} /* ForumMessage() */
	/**
	 * @see com.javacorporate.dbobj.SecuredDBObject
	 */
	public ForumMessage(String theUser) throws DBException {

		super(theUser);
	} /* ForumMessage(String) */
	/**
	 * Extends the usual add method to fetch a next number field
	 * and set the created date/time
	 *
	 * @throws	DBException If the next number cannot be determined or
	 * 			the add fails
	 */
	public void add() throws DBException {
		String myName = new String(thisClass + "add()");
		
		if (getField("NextMessage").equals("")) {
			setField("NextMessage", "0");
		}
		if (getField("ReplyTo").equals("")) {
			setField("ReplyTo", "0");
		}

		ForumThread myThread = new ForumThread();
		myThread.setField("ThreadId", getField("ThreadId"));
		myThread.retrieve();
		myThread.setField("LastPosting", DateTime.getDateTimeForDB());
		myThread.setField("LastPostingBy", getField("UserName"));
		myThread.update();

		ForumGroup myGroup = new ForumGroup();
		myGroup.setField("GroupId",  myThread.getField("GroupId"));
		myGroup.retrieve();
		myGroup.setField("LastPosting", DateTime.getDateTimeForDB());
		myGroup.setField("LastPostingBy", getField("UserName"));
		myGroup.update();
		
		NextNumber myNext = new NextNumber("SYSTEM");
		long myIdNum = myNext.getNextTentative("FMSG");
		setField("MessageId", new String().valueOf(myIdNum));

		Calendar rightNow = Calendar.getInstance();
		setField("Created", DateTime.getDateTimeForDB());
		super.add();
		myNext.getNextConfirm("FMSG");
		String replyString = null;

		/* Now find everyone who is set up for email notifications to this group and send them */
		/* the content of this message */
		try {
			ForumSubscribe fl = new ForumSubscribe();
			ForumSubscribe oneSubscribe = null;
			fl.setField("GroupId", myGroup.getField("GroupId"));
			fl.setField("EMail", "Y");
			for (Enumeration e = fl.searchAndRetrieve().elements(); e.hasMoreElements(); ) {
				oneSubscribe = (ForumSubscribe) e.nextElement();

				replyString = new String("\n\nMessage posted by user " + getField("UserName") 
					+ " at " + getField("Created") + "\n\nTo reply to this message: http://www.javacorporate.com/servlet/TrxActionServlet"
					+ "?trx=com.javacorporate.forum.trx.Browse&state=promptReply&MessageId=" + getField("MessageId")
					+ "&next=/components/forum/jsp/reply.jsp");
				EventHandler.notify(oneSubscribe.getField("UserName"), "[" + myGroup.getField("GroupName") + "] " 
					+ getField("Subject"), getField("Body") + replyString); 
			}
		} catch (LogException de) {
			LogHandler.tryLog(myName, "WARNING: Unable to send message from forum:" + de.getMessage(), "YELLOW"); 
		}
	} /* add() */
	/**
	 * @see com.javacorporate.dbobj.SecuredDBObject#getThisDBObj()	 
	 *
	 * @returns	DBObject
	 * @throws	DBException
	 */
	public DBObject getThisDBObj() throws DBException {
		return new ForumMessage();
	} /* getThisDBObj() */
	/**
	 * @see com.javacorporate.dbobj.SecuredDBObject
	 */
	protected synchronized void setupFields() throws DBException {
		setTargetTable("FORUMMESSAGE");

		setDescription("Forum Messages");

		addField("MessageId", "int", 0, false, "Message Id");
		addField("ThreadId", "int", 0, false, "Thread");
		addField("Subject", "varchar", 80, false, "Subject");
		addField("Body", "text", 0, true, "Body");
		addField("Created", "datetime", 0, false, "Created");
		addField("Status", "int", 0, false, "Status");
		addField("UserName", "char", 30, false, "User Name");
		addField("NextMessage", "int", 0, false, "Next Message");
		addField("ReplyTo", "int", 0, false, "Reply To");

		addKey("MessageId");
		setReadOnly("MessageId");
		setReadOnly("Created");
		//setMultiValued("Status");
	} /* setupFields() */
} /* ForumMessage */
